#!/bin/bash

# configuration des fichiers de logs
#  LogFile = fichier généré par ce test
#  RefFile = fichier de référence pour la comparaison finale.
LogFile="./logs/debugLog1.txt"
RefFile="./logs/debugLogTest1.txt"
echo "Fichier de log $LogFile"
echo "Fichier de référence $RefFile"


listPidToKill=""

# nettoyage d'éventuelles trace d'anciens tests
./scripts/killArbre.sh
rm $LodFile 2> /dev/null


# nettoyage en cas de kill ou ^C
trap 'echo mort de $listPidToKill ; kill $listPidToKill 2> /dev/null; rm tubeNode* 2> /dev/null; exit 1' INT TERM


echo "Lancement des processus avant scénario (qq secondes)"

############  Lancement commun à tous les tests : Debug + TaskServer
./scripts/Debug.sh $LogFile &
debug=$!
listPidToKill="$listPidToKill $debug"

rm tubeNode* 2> /dev/null

# serveur de taches
echo "Lancement du serveur de tâches sur le port 4545"
./scripts/TaskServer.sh 4545 &
taskServer=$!
listPidToKill="$listPidToKill $taskServer"

# on attend le démarage du serveur
sleep 1

############ mise en place de l'arbre de Node pour le test

# en premier tous les noeuds
# noeud racine
echo "Lancement du noeud racine (num:1, port:4501)"
./scripts/Node.sh 1 4501 localhost 4545  &
node1=$!
mkfifo tubeNode1
listPidToKill="$listPidToKill $node1"

# on attend le démarage du noeud 1
sleep 1


# ensuite une console par noeud
echo "Lancement des consoles"

./scripts/Console.sh 1 localhost 4501 < tubeNode1 &
console1=$!
listPidToKill="$listPidToKill $console1"


########### script du scénario #############
echo "Début du scénario prenant 1 secondes pour ce test"

# première demande: tache de 1 seconde sur noeud 1
echo 1 > tubeNode1

echo "le noeud 1 veut la ressource pour 1 seconde"

# durée des taches = 1 seconde . Durée déjà écoulée 0 seconde
# Durée restant 1 seconde. On s'endort 3 secondes

sleep 3 &

wait $!

###########  fin de la simulation #########

echo "fin des processus lancés: $listPidToKill"
kill $listPidToKill 2> /dev/null
rm tubeNode* 2>/dev/null

# description du test avant d'afficher son statut finale
cat << EOF
Scénario du test:
Test avec un seul noeud demandant une fois la ressource.
Arbre:
     1(T)

Temps 0: 1 demande la ressource pour 1 seconde

Résultat du test:
EOF
if ! diff -q $LogFile $RefFile > /dev/null ; then
    echo "Échec du test pour ce scénario"
    echo "$LogFile diffère de $RefFile"
    exit 1
else
    echo "Test OK"
fi
exit 0
