#!/bin/bash

# configuration des fichiers de logs
#  LogFile = fichier généré par ce test
#  RefFile = fichier de référence pour la comparaison finale.
LogFile="./logs/debugLog2.txt"
RefFile="./logs/debugLogTest2.txt"
echo "Fichier de log $LogFile"
echo "Fichier de référence $RefFile"


listPidToKill=""

# nettoyage d'éventuelles trace d'anciens tests
./scripts/killArbre.sh
rm $LodFile 2> /dev/null


# nettoyage en cas de kill ou ^C
trap 'echo mort de $listPidToKill ; kill $listPidToKill 2> /dev/null; rm tubeNode* 2> /dev/null; exit 1' INT TERM


echo "Lancement des processus avant scénario (qq secondes)"

############  Lancement commun à tous les tests : Debug + TaskServer
./scripts/Debug.sh $LogFile &
debug=$!
listPidToKill="$listPidToKill $debug"

rm tubeNode* 2> /dev/null

# serveur de taches
echo "Lancement du serveur de tâches sur le port 4545"
./scripts/TaskServer.sh 4545 &
taskServer=$!
listPidToKill="$listPidToKill $taskServer"

# on attend le démarage du serveur
sleep 1

############ mise en place de l'arbre de Node pour le test

# en premier tous les noeuds
# noeud racine
echo "Lancement du noeud racine (num:1, port:4501)"
./scripts/Node.sh 1 4501 localhost 4545  &
node1=$!
mkfifo tubeNode1
listPidToKill="$listPidToKill $node1"

# on attend le démarage du noeud 1
sleep 2

# noeud 2 fils de noeud 1
echo "Lancement du noeud 2 fils de noeud 1 (num:2, port:4502)"
./scripts/Node.sh 2 4502 localhost 4545 1 localhost 4501 &
node2=$!
mkfifo tubeNode2
listPidToKill="$listPidToKill $node2"

# on attend le démarage du noeud 2
sleep 1


# ensuite une console par noeud
echo "Lancement des consoles"

./scripts/Console.sh 1 localhost 4501 < tubeNode1 &
console1=$!
listPidToKill="$listPidToKill $console1"

./scripts/Console.sh 2 localhost 4502 < tubeNode2 &
console2=$!
listPidToKill="$listPidToKill $console2"

# on attend le démarage des consoles
sleep 1
########### script du scénario #############
echo "Début du scénario prenant 1 secondes pour ce test"

# première demande: tache de 1 secondes sur noeud 1
echo 2 > tubeNode1

echo "le noeud 1 veut la ressource pour 2 secondes"

# durée des taches = 2 secondes . Durée déjà écoulée 0 seconde
# Durée restant 2 secondes. On s'endort 4 secondes

sleep 4 &

wait $!

###########  fin de la simulation #########

echo "fin des processus lancés: $listPidToKill"
kill $listPidToKill 2> /dev/null
rm tubeNode* 2>/dev/null

# description du test avant d'afficher son statut finale
cat << EOF
Scénario du test:
Test avec deux noeuds. La racine (1) demande la ressource.
Arbre
     1(T)
    /
   2
Temps 0: 1 demande la ressource pour 2 secondes.

Résultat du test:
EOF

if ! diff -q $LogFile $RefFile > /dev/null ; then
    echo "Échec du test pour ce scénario"
    echo "$LogFile diffère de $RefFile"
    exit 1
else
    echo "Test OK"
fi
exit 0
