#!/bin/bash

# configuration des fichiers de logs
#  LogFile = fichier généré par ce test
#  RefFile = fichier de référence pour la comparaison finale.
LogFile="./logs/debugLog6.txt"
RefFile="./logs/debugLogTest6.txt"
echo "Fichier de log $LogFile"
echo "Fichier de référence $RefFile"


listPidToKill=""

# nettoyage d'éventuelles trace d'anciens tests
./scripts/killArbre.sh
rm $LodFile 2> /dev/null


# nettoyage en cas de kill ou ^C
trap 'echo mort de $listPidToKill ; kill $listPidToKill 2> /dev/null; rm tubeNode* 2> /dev/null; exit 1' INT TERM


echo "Lancement des processus avant scénario (qq secondes)"

############  Lancement commun à tous les tests : Debug + TaskServer
./scripts/Debug.sh $LogFile &
debug=$!
listPidToKill="$listPidToKill $debug"

rm tubeNode* 2> /dev/null

# serveur de taches
echo "Lancement du serveur de tâches sur le port 4545"
./scripts/TaskServer.sh 4545 &
taskServer=$!
listPidToKill="$listPidToKill $taskServer"

# on attend le démarage du serveur
sleep 1

############ mise en place de l'arbre de Node pour le test

############ mise en place de l'arbre de Node pour le test

# en premier tous les noeuds
# noeud racine
echo "Lancement du noeud racine (num:1, port:4501)"
./scripts/Node.sh 1 4501 localhost 4545  &
node1=$!
mkfifo tubeNode1
listPidToKill="$listPidToKill $node1"

# on attend le démarage du noeud 1
sleep 2

# noeud 2 fils de noeud 1
echo "Lancement du noeud 2 fils de noeud 1 (num:2, port:4502)"
./scripts/Node.sh 2 4502 localhost 4545 1 localhost 4501 &
node2=$!
mkfifo tubeNode2
listPidToKill="$listPidToKill $node2"

# on attend le démarage du noeud 2
sleep 1

# noeud 3 fils de noeud 1
echo "Lancement du noeud 3 fils de noeud 1 (num:3, port:4503)"
./scripts/Node.sh 3 4503 localhost 4545 1 localhost 4501 &
node3=$!
mkfifo tubeNode3
listPidToKill="$listPidToKill $node3"

# on attend le démarage du noeud 3
sleep 1

# ensuite une console par noeud
echo "Lancement des consoles"

./scripts/Console.sh 1 localhost 4501 < tubeNode1 &
console1=$!
listPidToKill="$listPidToKill $console1"

./scripts/Console.sh 2 localhost 4502 < tubeNode2 &
console2=$!
listPidToKill="$listPidToKill $console2"

./scripts/Console.sh 3 localhost 4503 < tubeNode3 &
console3=$!
listPidToKill="$listPidToKill $console3"


########### script du scénario #############
echo "Début du scénario prenant 12 secondes pour ce test"

# première demande: tache de 4 secondes sur noeud 1
echo 4 > tubeNode1
echo "le noeud 1 veut la ressource pour 4 secondes"
sleep 1


# deuxiéme demande: tache de 5 secondes sur noeud 2
echo 10 > tubeNode2
echo "le noeud 2 veut la ressource pour 10 secondes"
sleep 7

# troisiéme demande: tache de 2 secondes sur noeud 3
echo 2 > tubeNode3
echo "le noeud 3 veut la ressource pour 2 secondes"


# durée des taches = 16 secondes . Durée déjà écoulée 8 secondes
# Durée restant 8 secondes. On s'endort 10 secondes

sleep 10 &

wait $!

###########  fin de la simulation #########

echo "fin des processus lancés: $listPidToKill"
kill $listPidToKill 2> /dev/null
rm tubeNode* 2>/dev/null

# description du test avant d'afficher son statut finale
cat << EOF
Scénario du test:
Test avec trois noeuds demandant chacun une fois la ressource.
Arbre:
     1(T)
    / \\
   2   3
Temps 0: 1 demande la ressource pour 4 secondes
Temps 1: 2 demande la ressource pour 10 secondes
Temps 8: 3 demande la ressource pour 2 seconde2

Résultat du test:
EOF

if ! diff -q $LogFile $RefFile ; then
    echo "Échec du test pour ce scénario"
    echo "$LogFile diffère de $RefFile"
    exit 1
else
    echo "Test OK"
fi
exit 0
