package tsp.csc4509.appli;

import static tsp.csc4509.common.Log.COMM;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.InetSocketAddress;

import java.nio.channels.SocketChannel;
import java.nio.charset.Charset;

import org.apache.logging.log4j.Level;

import tsp.csc4509.common.Log;
import tsp.csc4509.tcpnio.FullDuplexMsgWorker;
import tsp.csc4509.tcpnio.ReadMessageStatus;
import tsp.csc4509.tpnote.TaskRequest;
import tsp.csc4509.tpnote.MessType;
import tsp.csc4509.tpnote.TaskResponse;

public class Console {
	/**
	 * number of expected arguments of main method.
	 */
	private static final int NBARGS = 3;
	/**
	 * 
	 */
	private static final int NODENUMARG = 0;
	/**
	 * position of the hostname in argv[].
	 */
	private static  final int HOSTARG = 1;
	/**
	 * position of the port number in argv[].
	 */
	private static  final int PORTARG = 2;

	/**
	 * Programme principale de la console.
	 * @param argv
	 * 			trois arguments:
	 * 			argv[0] : numéro du noeud.
	 * 			argv[1] : host du noeud.
	 * 			argv[2] : port du noeud. 
	 * @throws IOException
	 */
	public static void main(final String[] argv) throws IOException {
		InetAddress destAddr;
		InetSocketAddress destSockAddr;
		FullDuplexMsgWorker worker;


		if (argv.length != NBARGS) {
			System.out.println("usage: java Console <numéro du noeud> <machine du noeud> <port du noeud> ");
			return;
		}
		Log.setLevel(COMM, Level.WARN);
		int nodeNum = Integer.parseInt(argv[NODENUMARG]);
		
		destAddr = InetAddress.getByName(argv[HOSTARG]);
		destSockAddr = new InetSocketAddress(destAddr,
				Integer.parseInt(argv[PORTARG]));
		worker = new FullDuplexMsgWorker(SocketChannel.open(destSockAddr));

		BufferedReader bufin = new BufferedReader(new InputStreamReader(System.in, Charset.defaultCharset()));
		while (!Thread.interrupted()) {
			String consoleMsg = null;
			System.out.println("[Console " + nodeNum + "] Donnez la durée de la tâche (en seconde):");
			consoleMsg = bufin.readLine();
			if (consoleMsg == null) {
				break;
			}
			try {
				int delay = Integer.parseInt(consoleMsg);
				TaskRequest taskResuest = new TaskRequest(delay, nodeNum);
				worker.sendMsg(MessType.TASKREQUESTTYPE.ordinal(), taskResuest);
				ReadMessageStatus status = ReadMessageStatus.ReadUnstarted;
				while ((status != ReadMessageStatus.ReadDataCompleted) &&
						(status != ReadMessageStatus.ChannelClosed)) {
					status = worker.readMessage();
				}
				if (status == ReadMessageStatus.ReadDataCompleted) {
					if (worker.getMessType() == MessType.TASKRESPONSETYPE.ordinal()) {
						TaskResponse taskResponse = (TaskResponse) worker.getData();
						System.out.println("La tache est terminée, réponse: " + taskResponse);
					}
				}

			} catch (NumberFormatException e) {
				System.err.println("La console ne peut lire que des entiers (durée de la tache en seconde");

			}
		}




	}
}
