package tsp.csc4509.debug;

import static tsp.csc4509.common.Log.GEN;

import java.io.IOException;
import java.io.PrintStream;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.StandardSocketOptions;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;


import tsp.csc4509.tcpnio.FullDuplexMsgWorker;
import tsp.csc4509.tcpnio.ReadMessageStatus;

import tsp.csc4509.tpnote.Node;


public class DebugTools {
	private final static int PORT = 8888; 
	private final static String HOST = "localhost";
	private final static int MESSTYPE = 0;
	private final static int INFO_NODE_TYPE = 0;
	private final static int INFO_NODE_NUM = 1;
	private final static int INFO_LINK_NUM = 2;
	private final static int INFO_TOKEN = 3;
	private final static int INFO_REQUESTING = 4;
	private final static String FATHERTYPE = "father";
	private final static String NEXTTYPE = "next";



	static boolean lastHavingToken = false;

	private static void changeToken(final boolean havingToken) throws InterruptedException {
		if ((!lastHavingToken) && (havingToken)) {
			Thread.sleep(1000);
		}
		lastHavingToken = havingToken;
	}
	private static void sendToDebugLog(String message)  {
		InetAddress destAddr;
		try {
			destAddr = InetAddress.getByName(HOST);

			InetSocketAddress sockAddr = new InetSocketAddress(destAddr, PORT);
			FullDuplexMsgWorker worker = new FullDuplexMsgWorker(SocketChannel.open(sockAddr));

			worker.sendMsg(MESSTYPE, message);
			worker.close();
		} catch (IOException e) {
			return;
		}
	}

	private static void sendFather(Node node) throws InterruptedException  {
		changeToken(node.isHavingToken());
		if (node.getFather() != null) {
			sendToDebugLog(FATHERTYPE + ":" + node.getId().getNum() + ":" + node.getFather().getNum() + ":" + node.isHavingToken() + ":" + node.isRequesting());
		} else {
			sendToDebugLog(FATHERTYPE + ":" + node.getId().getNum() + ":-1:" + node.isHavingToken() + ":" + node.isRequesting());
		}
	}

	private static void sendNext(Node node) throws InterruptedException {
		changeToken(node.isHavingToken());
		if (node.getNext() != null) {
			sendToDebugLog(NEXTTYPE + ":" + node.getId().getNum() + ":" +  node.getNext().getNum()+ ":" + node.isHavingToken() + ":" + node.isRequesting());
		} else {
			sendToDebugLog(NEXTTYPE + ":" + node.getId().getNum() + ":-1:" + node.isHavingToken() + ":" + node.isRequesting());
		}
	}
	
	public  static void sendToLog(Node node) throws InterruptedException {
		sendFather(node);
		sendNext(node);
	}
	

	
	private DebugTools() {
		
	}

	public static void receiveLog(final PrintStream out) throws IOException {
		ServerSocketChannel listenChannel = ServerSocketChannel.open();
		listenChannel.setOption(StandardSocketOptions.SO_REUSEADDR, true);
		listenChannel.bind(new InetSocketAddress(PORT));
		Debug fatherDebug = new Debug();
		Debug nextDebug = new Debug();

		while (!Thread.interrupted()) {
			SocketChannel  rwChan = listenChannel.accept();
			FullDuplexMsgWorker worker = new FullDuplexMsgWorker(rwChan);
			ReadMessageStatus status = ReadMessageStatus.ReadUnstarted;
			while (status != ReadMessageStatus.ReadDataCompleted &&
					status != ReadMessageStatus.ChannelClosed) {
				status = worker.readMessage();
			}

			if (status == ReadMessageStatus.ReadDataCompleted) {
				if (worker.getMessType() == MESSTYPE) {
					String message = (String)  worker.getData();
					String [] infos = message.split(":");
					int nodeNum = Integer.parseInt(infos[INFO_NODE_NUM]);
					int fatherNumt = Integer.parseInt(infos[INFO_LINK_NUM]);
					boolean havingToken = infos[INFO_TOKEN].equals(""+true);
					boolean requesting = infos[INFO_REQUESTING].equals(""+true);
					
					GEN.info("Debug: " + infos[INFO_NODE_TYPE] + ":" + nodeNum + ":"+ fatherNumt + ":" + havingToken + ":" + requesting);
					switch (infos[INFO_NODE_TYPE]) {
					case FATHERTYPE:
						if (fatherDebug.changeLink(nodeNum, fatherNumt, havingToken, requesting)) {
							out.println("fathers:" + fatherDebug);
						}
						break;
					case NEXTTYPE:
						if (nextDebug.changeLink(nodeNum, fatherNumt, havingToken, requesting)) {
							out.println("nexts:" + nextDebug );
						}
						break;

					}


				}
			}
			worker.close();
		}
	}


}
