package tsp.csc4509.tpnote;

import java.net.InetSocketAddress;
import java.util.Objects;
import java.io.Serializable;
import java.net.InetAddress;

/**
 * Classe définissant identifiant les noeuds du graphe.
 * @author Éric Lallet
 *
 */
public class NodeId implements Serializable {
	private static final long serialVersionUID = 1L;

	/**
	 * adresse TCP/IP du noeud.
	 */
	private InetSocketAddress addr;
	
	/**
	 * numéro du noeud.
	 */
	private int num;

	/**
	 * Constructeur à partir d'une InetSocketAddress déjà complétée.
	 * @param addr
	 *          l'adresse  InetSocketAddress déjà complétée.
	 * @param num
	 *          numéro du noeud.
	 */
	public NodeId(final InetSocketAddress addr, final int num) {
		super();
		this.addr = addr;
		this.num = num;
	}
	
	
	
	/**
	 * Constructeur à partir d'une adresse InetAddress et un numéro de port.
	 * @param inetAddr
	 *            l'adresse IP sous forme InetAddress.
	 * @param port
	 *            le numéro de port TCP.
	 * @param num
	 *          numéro du noeud.
	 */
	public NodeId(InetAddress inetAddr, final int port, final int num) {
		super();
		this.addr = new InetSocketAddress(inetAddr, port);
		this.num = num;
	}

	
	/**
	 * getter sur l'adresse TCP/IP du noeud.
	 * @return
	 *        l'adresse TCP/IP du noeud.
	 */
	public InetSocketAddress getAddr() {
		return addr;
	}

	
	/**
	 * getter sur le numéro du noeud.
	 * @return
	 *        le numéro du noeud.
	 */
	public int getNum() {
		return num;
	}

	


	@Override
	public String toString() {
		return "NodeId [" + num + "(" + addr +")]";
	}



	// l'unicité de l'id du noeud est garantie par son adresse TCP/IP
	// et nous utilisons que celle-ci pour comparer deux id.
	@Override
	public int hashCode() {
		return Objects.hash(addr);
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		NodeId other = (NodeId) obj;
		return Objects.equals(addr, other.addr);
	}
}
