package tsp.csc4509.tpnote;

import static tsp.csc4509.common.Log.GEN;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.StandardSocketOptions;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;

import tsp.csc4509.tcpnio.FullDuplexMsgWorker;
import tsp.csc4509.tcpnio.ReadMessageStatus;

public class TaskServer {
	/**
	 * nombre de millisecondes dans une seconde.
	 */
	private static final int MILLISECONDSINSECOND = 1000;
	/**
	 * ServerSocketChannel du serveur pour les accept().
	 */
	private ServerSocketChannel listenChannel;
	
	/**
	 * compteur de TaskRequest reçus.
	 */
	private int requestNumber;

	// Question 1  // 1 pt
	/**
	 * Contructeur qui prépare le serveur en ouvrant le canal pour les accept(). 
	 * @param port
	 *            pour occupé par le serveur.
	 * @throws IOException
	 *            toutes les IOException qui peuvent se produire lors de l'ouverture
	 *            et le bind du ServerSocketChannel. 
	 */
	public TaskServer(final int port) throws IOException {
		listenChannel = ServerSocketChannel.open();
		listenChannel.setOption(StandardSocketOptions.SO_REUSEADDR, true);
		listenChannel.bind(new InetSocketAddress(port));
		requestNumber = 0;
	}

	// Question 2  // 3 pt
	/**
	 * la méthode qui réalise la boucle du serveur une fois le ServerSocketChannel ouvert, lié
	 * et prêt pour les accept().
	 * @throws IOException
	 *                toutes les IOException qui peuvent se produire  des accept()
	 *                et les échanges réseau.
	 * @throws InterruptedException
	 *                exception qui peut se produire lors de sleep() du serveur.
	 */
	public void loop() throws IOException, InterruptedException {

		while (true) {
			GEN.info("TaskServer: en attente de connexion");
			SocketChannel  rwChan = listenChannel.accept();  // 0.5
			FullDuplexMsgWorker worker = new FullDuplexMsgWorker(rwChan);
			GEN.info("TaskServer: connexion au serveur");

			ReadMessageStatus status = ReadMessageStatus.ReadUnstarted;
			while (status != ReadMessageStatus.ReadDataCompleted 
					&& status != ReadMessageStatus.ChannelClosed) { // 1pt dont 0.5 pour close
				status = worker.readMessage();
			}
			
			if (status == ReadMessageStatus.ReadDataCompleted) {
				if (worker.getMessType() == MessType.TASKREQUESTTYPE.ordinal()) { 
					TaskRequest taskRequest = (TaskRequest) worker.getData(); // 0.5
					requestNumber++;
					GEN.info("TaskServer: tache numero " + requestNumber + " reçue de " + taskRequest.getNumNode() + ", durée:" + taskRequest.getTaskDelay());
					Thread.sleep(MILLISECONDSINSECOND * taskRequest.getTaskDelay());
					
					GEN.info("TaskServer: tache numero " + requestNumber + " terminée");
					
					TaskResponse taskResponse = new TaskResponse(requestNumber);
					worker.sendMsg(MessType.TASKRESPONSETYPE.ordinal(), taskResponse); // 0.5
					GEN.info("TaskServer: Réponse à tache numero " + requestNumber + " envoyée");
				}
			}
			worker.close(); // 0.5 
		}

	}

	

	//*************  getters ajoutés pour faciliter les tests *************/
	 
	/**
	 * 
	 * @return
	 *       le canal serveur du serveur de tâches.
	 */
	public ServerSocketChannel getListenChannel() {
		return listenChannel;
	}

	/**
	 * 
	 * @return
	 *       le nombre de requêtes déjà traités par le serveur.
	 */
	public int getRequestNumber() {
		return requestNumber;
	}

	/**
	 * 
	 * @param listenChannel
	 */
	public void setListenChannel(final ServerSocketChannel listenChannel) {
		this.listenChannel = listenChannel;
	}

	/**
	 * 
	 * @param requestNumber
	 */
	public void setRequestNumber(final int requestNumber) {
		this.requestNumber = requestNumber;
	}

	

}
