package tsp.csc4509.tpnote;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTrue;
import static tsp.csc4509.common.Log.COMM;
import static tsp.csc4509.common.Log.TEST;
import static tsp.csc4509.common.Log.GEN;



import java.io.IOException;
import java.io.Serializable;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.StandardSocketOptions;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.apache.logging.log4j.Level;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;

import tsp.csc4509.common.Log;
import tsp.csc4509.tcpnio.FullDuplexMsgWorker;
import tsp.csc4509.tcpnio.ReadMessageStatus;


public class TestNode {
	/**
	 * thread des alarms
	 */
	private Thread alarmThread = null;

	/**
	 * numéro du port utilisé par les serveur lancés pour les tests.
	 */
	private static final int TESTSERVERPORT = 40001;

	private static final int TASKDELAY = 1;

	private Node nodeUnderTest;

	@BeforeAll
	public static void initTest() {
		Log.setLevel(TEST, Level.INFO);
		Log.setLevel(GEN, Level.INFO);
	}

	@BeforeEach
	public void createNode() throws IOException {
		nodeUnderTest = new Node();
		nodeUnderTest.setInConnexions(new HashMap<SelectionKey, FullDuplexMsgWorker>());
		nodeUnderTest.setSelector(Selector.open());
		nodeUnderTest.setTaskRequest(new TaskRequest(TASKDELAY, 1));
	}

	@AfterEach
	public void deleteNode() throws IOException {
		if (nodeUnderTest!= null && nodeUnderTest.getListenChannel() != null && nodeUnderTest.getListenChannel().isOpen()) {
			nodeUnderTest.getListenChannel().close();
		}
		nodeUnderTest = null;
	}


	@Disabled
	@Test
	void testSendTaskToServer() throws Exception {		
		TEST.info("Test de Node::sendTaskToServer()");




		try (ServerSocketChannel canalTest = ServerSocketChannel.open();) {
			canalTest.setOption(StandardSocketOptions.SO_REUSEADDR, true);
			canalTest.bind(new InetSocketAddress(TESTSERVERPORT));

			InetSocketAddress serverAddr = new InetSocketAddress(InetAddress.getByName("localhost"), TESTSERVERPORT);
			nodeUnderTest.setServerAddr(serverAddr);

			nodeUnderTest.sendTaskToServer();
			Selector selector = nodeUnderTest.getSelector();
			Set<SelectionKey> allKeys = selector.keys();
			assertTrue(allKeys.size() == 1, "le canal reliée au server est enregistré auprès du selector");
			SelectionKey chanKey = null;
			for(SelectionKey key: allKeys) {
				chanKey = key;
			}
			Map<SelectionKey, FullDuplexMsgWorker> connexions = nodeUnderTest.getInConnexions();
			assertTrue(connexions.size() == 1, "un worker a été ajouté dans la Map inConnexions");
			assertTrue(connexions.get(chanKey) != null, "la clef enregistrée est dans la Map inConnexions");
			FullDuplexMsgWorker chanWorker = connexions.get(chanKey);
			assertTrue(chanWorker.getChannel() == chanKey.channel(), "le worker assodié à la clef dans la Map est bien celui créé à partir du canal enregistré");
			alarmStart(5000);
			SocketChannel rwChan = canalTest.accept();
			alarmStop();
			assertTrue(rwChan != null, "La méthode sendTaskToServer() se connecte au serveur");
			FullDuplexMsgWorker worker = new FullDuplexMsgWorker(rwChan);
			alarmStart(5000);
			ReadMessageStatus status = ReadMessageStatus.ReadUnstarted;
			while ((!Thread. currentThread().isInterrupted()) &&
					status != ReadMessageStatus.ReadDataCompleted &&
					status != ReadMessageStatus.ChannelClosed) {
				status = worker.readMessage();
			}
			alarmStop();
			assertTrue(status == ReadMessageStatus.ReadDataCompleted, "La methode sendTaskToServer() envoie un message au server");
			Serializable message = worker.getData();
			assertTrue(message instanceof TaskRequest, "Le message envoyé par sendTaskToServer()  est un TaskRequest");
			TaskRequest requete = (TaskRequest) message;
			assertTrue(requete.getTaskDelay() == TASKDELAY, "Le delay du TaskRequest est correct " ); 

		} 

	}

	@Disabled
	@Test
	public void testReceivingResourceRequest1() throws IOException, InterruptedException {
		TEST.info("Test 1 de Node::receivingResourceRequest():  cas du noeud racine ne demandant pas la ressouce.");
		try (ServerSocketChannel reqNodeChan = ServerSocketChannel.open();) {
			reqNodeChan.setOption(StandardSocketOptions.SO_REUSEADDR, true);
			reqNodeChan.bind(new InetSocketAddress(TESTSERVERPORT));

			InetSocketAddress testAddr = new InetSocketAddress(InetAddress.getByName("localhost"), TESTSERVERPORT);
			// noeud racine ne demandant pas le ressource (donc ayant le jeton).
			nodeUnderTest.setRequesting(false);
			nodeUnderTest.setHavingToken(true);
			NodeId nodeId = new NodeId(null, 1);
			nodeUnderTest.setId(nodeId);
			NodeId requestNodeId = new NodeId(testAddr, 2);
			ResourceRequest resourceRequest = new ResourceRequest(requestNodeId);

			alarmStart(5000);
			// appel de la méthode testée
			nodeUnderTest.receivingResourceRequest(resourceRequest);
			SocketChannel rwChan = reqNodeChan.accept();
			alarmStop();
			assertTrue(rwChan != null, "La méthode receivingResourceRequest() se connecte au Node émeteur de la requête ");
			FullDuplexMsgWorker worker = new FullDuplexMsgWorker(rwChan);
			alarmStart(5000);
			ReadMessageStatus status = ReadMessageStatus.ReadUnstarted;
			while ( (!Thread. currentThread().isInterrupted()) &&
					status != ReadMessageStatus.ReadDataCompleted &&
					status != ReadMessageStatus.ChannelClosed) {
				status = worker.readMessage();
			}
			alarmStop();
			assertTrue(status == ReadMessageStatus.ReadDataCompleted, "La methode receivingResourceRequest() envoie un message au Node émeteur de la requête");
			Serializable message = worker.getData();
			assertTrue(message instanceof Token, "Le message envoyé par receivingResourceRequest()  est un Token");
			assertFalse(nodeUnderTest.isHavingToken(), "Le node n'a plus le jeton");
			assertFalse(nodeUnderTest.isRequesting(), "Le node ne demande pas la ressource");
		}

	}

	@Disabled
	@Test
	public void testReceivingResourceRequest2() throws IOException, InterruptedException {
		TEST.info("Test 2 de Node::receivingResourceRequest(): cas du noeud racine ayant la ressouce");
		try (ServerSocketChannel reqNodeChan = ServerSocketChannel.open();) {
			reqNodeChan.setOption(StandardSocketOptions.SO_REUSEADDR, true);
			reqNodeChan.bind(new InetSocketAddress(TESTSERVERPORT));

			InetSocketAddress reqNodeAddr = new InetSocketAddress(InetAddress.getByName("localhost"), TESTSERVERPORT);
			// noeud racine ne demandant pas le ressource (donc ayant le jeton).
			nodeUnderTest.setRequesting(true);
			nodeUnderTest.setHavingToken(true);
			NodeId nodeId = new NodeId(null, 1);
			nodeUnderTest.setId(nodeId);
			NodeId reqNodeId = new NodeId(reqNodeAddr, 2);
			ResourceRequest resourceRequest = new ResourceRequest(reqNodeId);

			alarmStart(5000);
			// appel de la méthode testée
			nodeUnderTest.receivingResourceRequest(resourceRequest);
			SocketChannel rwChan = reqNodeChan.accept();
			alarmStop();
			assertTrue(rwChan != null, "La méthode receivingResourceRequest() se connecte une première fois au Node émeteur de la requête ");
			alarmStart(5000);
			rwChan = reqNodeChan.accept();
			alarmStop();
			assertTrue(rwChan != null, "La méthode receivingResourceRequest() se connecte une seconde fois au Node émeteur de la requête ");

			assertTrue(nodeUnderTest.getFather().equals(reqNodeId), "Le pere est le noeud émetteur de la requête");
			assertTrue(nodeUnderTest.getFatherConnexion() != null, "La connexion au père est ouverte");
			InetSocketAddress remoteAddr = (InetSocketAddress) nodeUnderTest.getFatherConnexion().getChannel().getRemoteAddress();
			assertTrue(remoteAddr.getPort() == TESTSERVERPORT, "la connexion père est bien vers le neoud émetteur de la requête");

			assertTrue(nodeUnderTest.getNext().equals(reqNodeId), "Le suivant est le noeud émetteur de la requête");
			assertTrue(nodeUnderTest.getNextConnexion() != null, "La connexion au suivant est ouverte");
			remoteAddr = (InetSocketAddress) nodeUnderTest.getNextConnexion().getChannel().getRemoteAddress();
			assertTrue(remoteAddr.getPort() == TESTSERVERPORT, "la connexion next est bien vers le neoud émetteur de la requête");

			assertTrue(nodeUnderTest.isHavingToken(), "Le node a toujours le jeton");
			assertTrue(nodeUnderTest.isRequesting(), "Le node utilise toujours la ressource");

		}

	}

	@Disabled
	@Test
	public void testReceivingResourceRequest3() throws IOException, InterruptedException {
		TEST.info("Test 3 de Node::receivingResourceRequest(): cas du noeud qui n'est pas à la racine");
		try (ServerSocketChannel reqNodeChan = ServerSocketChannel.open();
				ServerSocketChannel fatherNodeChan = ServerSocketChannel.open()) {
			// création d'un pseudo noeud pour l'émetteur de la requête
			reqNodeChan.setOption(StandardSocketOptions.SO_REUSEADDR, true);
			reqNodeChan.bind(new InetSocketAddress(TESTSERVERPORT));
			InetSocketAddress reqNodeAddr = new InetSocketAddress(InetAddress.getByName("localhost"), TESTSERVERPORT);
			NodeId reqNodeId = new NodeId(reqNodeAddr, 2);

			// création d'un pseudo noeud pour le père
			fatherNodeChan.setOption(StandardSocketOptions.SO_REUSEADDR, true);
			fatherNodeChan.bind(new InetSocketAddress(TESTSERVERPORT+1));
			InetSocketAddress fatherNodeAddr = new InetSocketAddress(InetAddress.getByName("localhost"), TESTSERVERPORT+1);
			NodeId fatherNodeId = new NodeId(fatherNodeAddr, 3);

			// création d'un fatherRwChan pour faire le lien father pour le nodeUnderTest
			SocketChannel fatherRwChan = SocketChannel.open(fatherNodeAddr); // accept()ci dessous
			FullDuplexMsgWorker fatherWorker = new FullDuplexMsgWorker(fatherRwChan); 

			// canal relié du coté de pseudo père
			SocketChannel sonRwChan = fatherNodeChan.accept(); // accept la connexion faite juste au dessus
			FullDuplexMsgWorker sonWorker = new FullDuplexMsgWorker(sonRwChan);


			// noeud qui n'est pas à la racine
			nodeUnderTest.setRequesting(false); // n'est pas utilisé et ne doit pas être changé par la méthode testée
			nodeUnderTest.setHavingToken(false);// n'est pas utilisé et ne doit pas être changé par la méthode testée
			nodeUnderTest.setFather(fatherNodeId);
			nodeUnderTest.setFatherConnexion(fatherWorker);
			NodeId nodeId = new NodeId(null, 1);
			nodeUnderTest.setId(nodeId);


			ResourceRequest resourceRequest = new ResourceRequest(reqNodeId);

			alarmStart(5000);
			// appel de la méthode testée
			nodeUnderTest.receivingResourceRequest(resourceRequest);

			alarmStart(5000);
			ReadMessageStatus status = ReadMessageStatus.ReadUnstarted;
			while (status != ReadMessageStatus.ReadDataCompleted &&
					status != ReadMessageStatus.ChannelClosed) {
				status = sonWorker.readMessage();
			}
			alarmStop();
			assertTrue(status == ReadMessageStatus.ReadDataCompleted, "La methode receivingResourceRequest() envoie un message à son père");
			Serializable message = sonWorker.getData();
			assertTrue(message instanceof ResourceRequest, "Le message envoyé par receivingResourceRequest()  est un ResourceRequest");
			ResourceRequest req = (ResourceRequest) message;
			assertTrue(req.equals(resourceRequest), "la requête émise par le noeud est celle qui lui a été envoyée");

			assertTrue(nodeUnderTest.getFather().equals(reqNodeId), "Le père du noeud est devenu le noeud émetteur de la requête");
			assertTrue(nodeUnderTest.getFatherConnexion() != null, "Le noeud est connecté à son père");
			InetSocketAddress remoteAddr = (InetSocketAddress) nodeUnderTest.getFatherConnexion().getChannel().getRemoteAddress();
			assertTrue(remoteAddr.getPort() == TESTSERVERPORT, "la connexion père est bien vers le noeud émetteur de la requête");

			assertFalse(nodeUnderTest.isHavingToken(), "Le boolean havingToken n'a pas changé");
			assertFalse(nodeUnderTest.isRequesting(), "Le boolean requesting n'a pas changé");

		}


	}


	@Disabled
	@Test
	public void testRequestingResource1() throws IOException, InterruptedException {
		TEST.info("Test 1 de Node::requestingResource():  recéption par un noeud pas racine.");
		try (ServerSocketChannel fatherNodeChan = ServerSocketChannel.open();) {

			// création d'un pseudo noeud pour le père
			fatherNodeChan.setOption(StandardSocketOptions.SO_REUSEADDR, true);
			fatherNodeChan.bind(new InetSocketAddress(TESTSERVERPORT+1));
			InetSocketAddress fatherNodeAddr = new InetSocketAddress(InetAddress.getByName("localhost"), TESTSERVERPORT+1);
			NodeId fatherNodeId = new NodeId(fatherNodeAddr, 3);

			// création d'un fatherRwChan pour faire le lien father pour le nodeUnderTest
			SocketChannel fatherRwChan = SocketChannel.open(fatherNodeAddr); // accept() ci dessous
			FullDuplexMsgWorker fatherWorker = new FullDuplexMsgWorker(fatherRwChan); 

			// canal relié du coté de pseudo père
			SocketChannel sonRwChan = fatherNodeChan.accept(); // accept la connexion faite juste au dessus
			FullDuplexMsgWorker sonWorker = new FullDuplexMsgWorker(sonRwChan);


			// noeud qui ne demande pas encore la ressourcee et qui n'est pas racine => pas de jeton
			nodeUnderTest.setRequesting(false);
			nodeUnderTest.setHavingToken(false);
			NodeId nodeId = new NodeId(null, 1);
			nodeUnderTest.setId(nodeId);
			nodeUnderTest.setFather(fatherNodeId);
			nodeUnderTest.setFatherConnexion(fatherWorker);

			TaskRequest taskRequest = new TaskRequest(1, 1);
			alarmStart(5000);
			// appel de la méthode testée
			nodeUnderTest.requestingResource(taskRequest);
			alarmStop();


			alarmStart(5000);
			ReadMessageStatus status = ReadMessageStatus.ReadUnstarted;
			while ( (!Thread. currentThread().isInterrupted()) &&
					status != ReadMessageStatus.ReadDataCompleted &&
					status != ReadMessageStatus.ChannelClosed) {
				status = sonWorker.readMessage();
			}
			alarmStop();

			assertTrue(status == ReadMessageStatus.ReadDataCompleted, "La methode requestingResource() envoie un message au Node père");
			Serializable message = sonWorker.getData();
			assertTrue(message instanceof ResourceRequest, "Le message envoyé par requestingResource()  est un ResourceRequest");
			ResourceRequest resourceRequest = (ResourceRequest) message;
			assertTrue(resourceRequest.getNodeId().equals(nodeId), "L'id dans ResourceRequest est l'id du Node");
			assertTrue(nodeUnderTest.getFather() == null, "Le Node n'a plus de père");
			assertTrue(nodeUnderTest.getFatherConnexion() == null, "Le Node n'a plus de connexion vers un père");
			assertFalse(fatherWorker.getChannel().isOpen(), "Le Node a fermer la connexion vers son (ancien) père");
			assertFalse(nodeUnderTest.isHavingToken(), "Le node n'a pas le jeton");
			assertTrue(nodeUnderTest.isRequesting(), "Le node demande la ressource");
			assertTrue(taskRequest.equals(nodeUnderTest.getTaskRequest()), "Le node a mémorisé la taskRequest en cours");
		}
	}

	@Disabled
	@Test
	public void testRequestingResource2() throws IOException, InterruptedException {
		TEST.info("Test 2 de Node::requestingResource():  recéption par un noeud racine.");
		try (ServerSocketChannel servChan = ServerSocketChannel.open();) {

			// création d'un pseudo noeud pour le père
			servChan.setOption(StandardSocketOptions.SO_REUSEADDR, true);
			servChan.bind(new InetSocketAddress(TESTSERVERPORT));
			InetSocketAddress servAddr = new InetSocketAddress(InetAddress.getByName("localhost"), TESTSERVERPORT);


			// noeud qui ne demande pas encore la ressource et qui est la racine => il a déjà le jeton
			nodeUnderTest.setRequesting(false);
			nodeUnderTest.setHavingToken(true);
			NodeId nodeId = new NodeId(null, 1);
			nodeUnderTest.setId(nodeId);
			nodeUnderTest.setServerAddr(servAddr);


			TaskRequest taskRequest = new TaskRequest(1, 1);
			alarmStart(5000);
			// appel de la méthode testée
			nodeUnderTest.requestingResource(taskRequest);
			SocketChannel rwChan = servChan.accept();
			alarmStop();

			assertTrue(rwChan != null, "Le node se connecte au serveur de ressource");
			FullDuplexMsgWorker worker = new FullDuplexMsgWorker(rwChan);

			alarmStart(5000);
			ReadMessageStatus status = ReadMessageStatus.ReadUnstarted;
			while ( (!Thread. currentThread().isInterrupted()) &&
					status != ReadMessageStatus.ReadDataCompleted &&
					status != ReadMessageStatus.ChannelClosed) {
				status = worker.readMessage();
			}
			alarmStop();

			assertTrue(status == ReadMessageStatus.ReadDataCompleted, "La methode requestingResource() envoie un message au server de ressource");
			Serializable message = worker.getData();
			assertTrue(message instanceof TaskRequest, "Le message envoyé par requestingResource()  est un TaskRequest");
			assertTrue(taskRequest.equals(nodeUnderTest.getTaskRequest()), "Le node a mémorisé la taskRequest en cours");

			assertTrue(nodeUnderTest.isHavingToken(), "Le node a encore le jeton");
			assertTrue(nodeUnderTest.isRequesting(), "Le node demande la ressource");
		}
	}

	@Disabled
	@Test
	public void testReceivingToken() throws IOException, InterruptedException {
		TEST.info("Test de Node::receivingToken():  recéption du token par un node.");
		try (ServerSocketChannel servChan = ServerSocketChannel.open();) {

			// création d'un pseudo noeud pour le père
			servChan.setOption(StandardSocketOptions.SO_REUSEADDR, true);
			servChan.bind(new InetSocketAddress(TESTSERVERPORT));
			InetSocketAddress servAddr = new InetSocketAddress(InetAddress.getByName("localhost"), TESTSERVERPORT);


			// noeud qui demande  la ressource mais qui n'a pas encore le jeton
			nodeUnderTest.setRequesting(true);
			nodeUnderTest.setHavingToken(false);
			NodeId nodeId = new NodeId(null, 1);
			nodeUnderTest.setId(nodeId);
			nodeUnderTest.setServerAddr(servAddr);
			TaskRequest taskRequest = new TaskRequest(1, 1);
			nodeUnderTest.setTaskRequest(taskRequest);



			alarmStart(5000);
			// appel de la méthode testée
			nodeUnderTest.receivingToken();
			SocketChannel rwChan = servChan.accept();
			alarmStop();

			assertTrue(rwChan != null, "Le node se connecte au serveur de ressource");
			FullDuplexMsgWorker worker = new FullDuplexMsgWorker(rwChan);

			alarmStart(5000);
			ReadMessageStatus status = ReadMessageStatus.ReadUnstarted;
			while ( (!Thread. currentThread().isInterrupted()) &&
					status != ReadMessageStatus.ReadDataCompleted &&
					status != ReadMessageStatus.ChannelClosed) {
				status = worker.readMessage();
			}
			alarmStop();

			assertTrue(status == ReadMessageStatus.ReadDataCompleted, "La methode receivingToken() envoie un message au server de ressource");
			Serializable message = worker.getData();
			assertTrue(message instanceof TaskRequest, "Le message envoyé par receivingToken()  est un TaskRequest");


			assertTrue(nodeUnderTest.isHavingToken(), "Le node a le jeton");
			assertTrue(nodeUnderTest.isRequesting(), "Le node demande la ressource");
		}
	}

	@Disabled
	@Test
	public void test1ReleasingResource() throws IOException, InterruptedException {
		TEST.info("Test 1 de Node::releasingResource():  libération de la ressource pour un noeud qui a un suivant.");
		try (ServerSocketChannel nextNodeChan = ServerSocketChannel.open();) {

			// création d'un pseudo noeud pour le next
			nextNodeChan.setOption(StandardSocketOptions.SO_REUSEADDR, true);
			nextNodeChan.bind(new InetSocketAddress(TESTSERVERPORT+1));
			InetSocketAddress nextAddr = new InetSocketAddress(InetAddress.getByName("localhost"), TESTSERVERPORT+1);
			NodeId nextNodeId = new NodeId(nextAddr, 3);

			// création d'un nextRwChan pour faire le lien next pour le nodeUnderTest
			SocketChannel nextRwChan = SocketChannel.open(nextAddr); // accept() ci dessous
			FullDuplexMsgWorker nextWorker = new FullDuplexMsgWorker(nextRwChan); 

			// canal relié du coté de pseudo next
			SocketChannel prevRwChan = nextNodeChan.accept(); // accept la connexion faite juste au dessus
			FullDuplexMsgWorker prevWorker = new FullDuplexMsgWorker(prevRwChan);


			// noeud qui libère le jeton avec un suivant dans la liste
			nodeUnderTest.setRequesting(true);
			nodeUnderTest.setHavingToken(true);
			NodeId nodeId = new NodeId(null, 1);
			nodeUnderTest.setId(nodeId);
			nodeUnderTest.setNext(nextNodeId);
			nodeUnderTest.setNextConnexion(nextWorker);


			alarmStart(5000);
			// appel de la méthode testée
			nodeUnderTest.releasingResource();

			ReadMessageStatus status = ReadMessageStatus.ReadUnstarted;
			while ( (!Thread. currentThread().isInterrupted()) &&
					status != ReadMessageStatus.ReadDataCompleted &&
					status != ReadMessageStatus.ChannelClosed) {
				status = prevWorker.readMessage();
			}
			alarmStop();

			assertTrue(status == ReadMessageStatus.ReadDataCompleted, "La methode releasingResource() envoie un message au suivant");
			Serializable message = prevWorker.getData();
			assertTrue(message instanceof Token, "Le message envoyé par releasingResource()  est un Token");
			assertTrue(nodeUnderTest.getNext() == null, "Le Node n'a plus de suivant");
			assertTrue(nodeUnderTest.getNextConnexion() == null, "Le Node n'a plus de connexion vers un suivant");
			assertFalse(nextWorker.getChannel().isOpen(), "Le Node a fermer la connexion vers son suivant");

			assertFalse(nodeUnderTest.isHavingToken(), "Le node n'a plus le jeton");
			assertFalse(nodeUnderTest.isRequesting(), "Le node ne demande plus la ressource");
		}
	}

	@Disabled
	@Test
	public void test2ReleasingResource() throws IOException, InterruptedException {
		TEST.info("Test 2 de Node::releasingResource():  libération de la ressource pour un noeud sans suivant");


		// noeud qui libère le jeton sans un suivant dans la liste
		// (il est donc dernier de la liste, donc racine de l'arbre)
		nodeUnderTest.setRequesting(true);
		nodeUnderTest.setHavingToken(true);
		NodeId nodeId = new NodeId(null, 1);
		nodeUnderTest.setId(nodeId);
		nodeUnderTest.setNext(null);
		nodeUnderTest.setNextConnexion(null);
		nodeUnderTest.setFather(null);
		nodeUnderTest.setFatherConnexion(null);

		alarmStart(5000);
		// appel de la méthode testée
		nodeUnderTest.releasingResource();
		alarmStop();

		assertTrue(nodeUnderTest.isHavingToken(), "Le node a toujours le jeton");
		assertFalse(nodeUnderTest.isRequesting(), "Le node ne demande plus la ressource");
		assertTrue(nodeUnderTest.getNext() == null, "Pas de next pour ce noeud");
		assertTrue(nodeUnderTest.getFather() == null, "Ce noeud est resté la racine de l'arbre");

	}

	@Disabled
	@Test
	public void testOpenListenChannel() throws IOException, InterruptedException {
		TEST.info("Test de Node::openListenChannel(): création du serveur asynchrone du noeud");

		// le noeud nodeUnderTest du @BeforeEach convient pour ce test

		// appel de la méthode testée
		nodeUnderTest.openListenChannel(TESTSERVERPORT+2);
		Selector selector = nodeUnderTest.getSelector();
		ServerSocketChannel listenChan = nodeUnderTest.getListenChannel();


		assertFalse(listenChan.isBlocking(), "le listenChanel est passé en asynchrone");

		Set<SelectionKey> allKeys = selector.keys();
		assertTrue(allKeys.size() == 1, "Il y a une clef enregistrée dans le selector");

		SelectionKey chanKey = null;
		for(SelectionKey key: allKeys) {
			chanKey = key;
		}
		assertTrue(chanKey.channel() == listenChan, "Le canal listenChan est enregistré dans le selctor");
		InetSocketAddress listenAddr = (InetSocketAddress) listenChan.getLocalAddress();
		assertTrue(listenAddr.getPort() == TESTSERVERPORT+2, "Le canal lisenChan est lié à TESTSERVERPORT");
	}




	/**
	 * Débute une alarme qui enverra une intérruption au thread courrant dans timeout milisecondes.
	 * Utiliser alarmStop() pour arrêter cette alarme.
	 * @param timeout
	 *           durée en millisecondes avant d'envoyer l'interruption.
	 */
	public void alarmStart(final long timeout) {
		Thread t = Thread.currentThread();

		Runnable alarmRun = new Runnable() {
			@Override
			public void run() {
				try {	
					TEST.trace("{}", () ->  "TcpTestTools: Lancement du service d'alarme pour une durée de " + timeout);
					Thread.sleep(timeout);
					t.interrupt();

				} catch (Exception e1) {
					// le sleep a été interrompu avant le delais: rien à faire.
					if (e1.getMessage() != null) {

						TEST.debug("{}", () -> "TcpTestTools:  Exception:" + e1.getMessage());

					}

				} 


				TEST.trace("TcpTestTools: Arret de l'alarme");
				alarmThread = null;


			}

		};
		alarmThread = new Thread(alarmRun);
		alarmThread.start();
	}

	/**
	 * Interrompt un Thread lancé précédemment. 
	 * @param t
	 *         référence sur le thread à interrompre.
	 * @param message
	 *         message de service qui annonce la fin du thread dans le log niveau trace.
	 *         Passer une chaîne null pour rendre la méthode muette.
	 * @throws InterruptedException
	 *         exception en cas d'interruption pendant join().
	 */
	private static void serviceStop(final Thread t, final String message) throws InterruptedException {

		if (message != null) {
			TEST.trace("TcpTestTools: " +  message);
		}

		if ((t != null) && (t.isAlive())) {
			t.interrupt();
			t.join();
		}


		if (message != null) {
			TEST.trace("TcpTestTools: Thread terminé");
		}


	}
	/**
	 * Interrompt le thread qui réalise le service alarm.
	 * @throws InterruptedException
	 *       exception en cas d'interruption pendant join().
	 */
	public void alarmStop() throws InterruptedException {
		serviceStop(alarmThread, "Interruption du service d'alarme");
	}

}

