package tsp.csc4509.tpnote;
import static org.junit.jupiter.api.Assertions.assertTrue;
import static tsp.csc4509.common.Log.TEST;

import java.io.IOException;
import java.io.Serializable;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.StandardSocketOptions;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;

import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;

import tsp.csc4509.tcpnio.FullDuplexMsgWorker;
import tsp.csc4509.tcpnio.ReadMessageStatus;


public class TestTaskServer {
	/**
	 * thread des alarms
	 */
	private Thread alarmThread = null;

	/**
	 * numéro du port utilisé par les serveur lancés pour les tests.
	 */
	private static final int TESTSERVERPORT = 40001;



	@Disabled
	@Test
	void testTaskServer() throws Exception {		
		TEST.info("Test du constructeur de TaskServer");

		TaskServer taskServer = new TaskServer(TESTSERVERPORT);
		assertTrue(taskServer.getListenChannel() != null, "le canal listenChannel a été instancieé");
		assertTrue (taskServer.getListenChannel().getLocalAddress() instanceof InetSocketAddress , "le canal a été lié une adresse TCP/IP");
		InetSocketAddress inetAddr = (InetSocketAddress) taskServer.getListenChannel().getLocalAddress();
		assertTrue(inetAddr.getPort() == TESTSERVERPORT, "Le canal a bien été lié au port attendu: " + TESTSERVERPORT);
	}

	@Disabled
	@Test
	void testLoop() throws IOException, InterruptedException {
		
		TEST.info("Test de TaskServer::loop()");
		final int TASKNUMBER = 10;
		final int TASKDELAY = 1;
		
		Thread testedServer = new Thread( () -> {
			TaskServer taskServer;
			try {
				taskServer = new TaskServer(TESTSERVERPORT);

				// on oublie ce qu'a fait le constructeur. On ne veut pas supposer qu'il est correctement
				// écrit pour ce test.
				if (taskServer.getListenChannel() != null &&  taskServer.getListenChannel().isOpen()) {
					taskServer.getListenChannel().close();
				}
			} catch (IOException e) {
				return;
			}
			try (ServerSocketChannel canalTest = ServerSocketChannel.open();) {
				canalTest.setOption(StandardSocketOptions.SO_REUSEADDR, true);
				canalTest.bind(new InetSocketAddress(TESTSERVERPORT+1));
				taskServer.setListenChannel(canalTest);
				taskServer.setRequestNumber(TASKNUMBER);
				taskServer.loop();
			} catch (IOException | InterruptedException e) {
				return;
			}
		});

		testedServer.start();
		Thread.sleep(1000);

		SocketChannel client = SocketChannel.open(new InetSocketAddress(InetAddress.getByName("localhost"), TESTSERVERPORT+1));
		FullDuplexMsgWorker worker = new FullDuplexMsgWorker(client);
		TaskRequest taskRequest = new TaskRequest(TASKDELAY, 1);
		worker.sendMsg(MessType.TASKREQUESTTYPE.ordinal(), taskRequest);
		alarmStart((TASKDELAY + 5) * 1000);
		ReadMessageStatus status = ReadMessageStatus.ReadUnstarted;
		while (status != ReadMessageStatus.ReadDataCompleted &&
				status != ReadMessageStatus.ChannelClosed) {
			status = worker.readMessage();
		}
		alarmStop();
		assertTrue(status == ReadMessageStatus.ReadDataCompleted, "La methode loop() répond à temps au TaskResquest");
		Serializable message = worker.getData();
		assertTrue(message instanceof TaskResponse, "Le message envoyé par loop() est un TaskResponse");
		TaskResponse reponse = (TaskResponse) message;
		assertTrue(reponse.getResult() == TASKNUMBER+1, "Le résultat dans la réponse est correct " ); 
	}


	/**
	 * Débute une alarme qui enverra une intérruption au thread courrant dans timeout milisecondes.
	 * Utiliser alarmStop() pour arrêter cette alarme.
	 * @param timeout
	 *           durée en millisecondes avant d'envoyer l'interruption.
	 */
	public void alarmStart(final long timeout) {
		Thread t = Thread.currentThread();

		Runnable alarmRun = new Runnable() {
			@Override
			public void run() {
				try {	
					TEST.trace("{}", () ->  "TcpTestTools: Lancement du service d'alarme pour une durée de " + timeout);
					Thread.sleep(timeout);
					t.interrupt();

				} catch (Exception e1) {
					if (e1.getMessage() != null) {

						TEST.debug("{}", () -> "TcpTestTools:  Exception:" + e1.getMessage());

					}
					// fin du thread sans autre explication... TODO traiter les fins anormales!
				} 


				TEST.trace("TcpTestTools: Fin du service client send file");
				alarmThread = null;


			}

		};
		alarmThread = new Thread(alarmRun);
		alarmThread.start();
	}
	
	/**
	 * Interrompt un Thread lancé précédemment. 
	 * @param t
	 *         référence sur le thread à interrompre.
	 * @param message
	 *         message de service qui annonce la fin du thread dans le log niveau trace.
	 *         Passer une chaîne null pour rendre la méthode muette.
	 * @throws InterruptedException
	 *         exception en cas d'interruption pendant join().
	 */
	private static void serviceStop(final Thread t, final String message) throws InterruptedException {
		
			if (message != null) {
				TEST.trace("TcpTestTools: " +  message);
			}
		
		if ((t != null) && (t.isAlive())) {
			t.interrupt();
			t.join();
		}
		
		
			if (message != null) {
				TEST.trace("TcpTestTools: Thread terminé");
			}
		
		
	}
	/**
	 * Interrompt le thread qui réalise le service alarm.
	 * @throws InterruptedException
	 *       exception en cas d'interruption pendant join().
	 */
	public void alarmStop() throws InterruptedException {
		serviceStop(alarmThread, "Interruption du service d'alarme");
	}

}

